@@ -3,14 +3,13 @@ import os
 from subprocess import Popen, PIPE
 import unittest
 
-import scrapy
+from scrapy.utils.test import get_pythonpath
 
 class CmdlineTest(unittest.TestCase):
 
     def setUp(self):
         self.env = os.environ.copy()
-        self.env['PYTHONPATH'] = os.path.dirname(scrapy.__path__[0])
-        self.env.pop('SCRAPY_SETTINGS_DISABLED', None)
+        self.env['PYTHONPATH'] = get_pythonpath()
         self.env['SCRAPY_SETTINGS_MODULE'] = 'scrapy.tests.test_cmdline.settings'
 
     def _execute(self, *new_args, **kwargs):
@@ -21,14 +20,14 @@ class CmdlineTest(unittest.TestCase):
 
     def test_default_settings(self):
         self.assertEqual(self._execute('settings', '--get', 'TEST1'), \
-            'default + loaded + started')
+            'default + started')
 
     def test_override_settings_using_set_arg(self):
-        self.assertEqual(self._execute('settings', '--get', 'TEST1', '--set', 'TEST1=override'), \
-            'override + loaded + started')
+        self.assertEqual(self._execute('settings', '--get', 'TEST1', '-s', 'TEST1=override'), \
+            'override + started')
 
     def test_override_settings_using_envvar(self):
         self.env['SCRAPY_TEST1'] = 'override'
         self.assertEqual(self._execute('settings', '--get', 'TEST1'), \
-            'override + loaded + started')
+            'override + started')
 
