@@ -17,8 +17,7 @@ from twisted.web import server, static, util
 from twisted.trial import unittest
 
 from scrapy import signals
-from scrapy.conf import Settings
-from scrapy.crawler import Crawler
+from scrapy.utils.test import get_crawler
 from scrapy.xlib.pydispatch import dispatcher
 from scrapy.tests import tests_datadir
 from scrapy.spider import BaseSpider
@@ -91,15 +90,14 @@ class CrawlerRun(object):
         for name, signal in vars(signals).items():
             if not name.startswith('_'):
                 dispatcher.connect(self.record_signal, signal)
-        dispatcher.connect(self.item_scraped, signals.item_scraped)
-        dispatcher.connect(self.request_received, signals.request_received)
-        dispatcher.connect(self.response_downloaded, signals.response_downloaded)
 
-        settings = Settings()
-        self.crawler = Crawler(settings)
+        self.crawler = get_crawler()
         self.crawler.install()
         self.crawler.configure()
-        self.crawler.queue.append_spider(self.spider)
+        self.crawler.signals.connect(self.item_scraped, signals.item_scraped)
+        self.crawler.signals.connect(self.request_received, signals.request_received)
+        self.crawler.signals.connect(self.response_downloaded, signals.response_downloaded)
+        self.crawler.crawl(self.spider)
         self.crawler.start()
 
         self.deferred = defer.Deferred()
@@ -199,6 +197,7 @@ class EngineTest(unittest.TestCase):
                          self.run.signals_catched[signals.spider_opened])
         self.assertEqual({'spider': self.run.spider},
                          self.run.signals_catched[signals.spider_idle])
+        self.run.signals_catched[signals.spider_closed].pop('spider_stats', None) # XXX: remove for scrapy 0.17
         self.assertEqual({'spider': self.run.spider, 'reason': 'finished'},
                          self.run.signals_catched[signals.spider_closed])
 
