@@ -1,8 +1,7 @@
 import unittest
-import weakref
 
-from scrapy.http import Response, TextResponse, HtmlResponse, XmlResponse, Headers
-from scrapy.utils.encoding import resolve_encoding
+from w3lib.encoding import resolve_encoding
+from scrapy.http import Request, Response, TextResponse, HtmlResponse, XmlResponse, Headers
 
 
 class BaseResponseTest(unittest.TestCase):
@@ -26,15 +25,11 @@ class BaseResponseTest(unittest.TestCase):
 
         assert isinstance(r.headers, Headers)
         self.assertEqual(r.headers, {})
-        self.assertEqual(r.meta, {})
 
-        meta = {"lala": "lolo"}
         headers = {"caca": "coco"}
         body = "a body"
-        r = self.response_class("http://www.example.com", meta=meta, headers=headers, body=body)
+        r = self.response_class("http://www.example.com", headers=headers, body=body)
 
-        assert r.meta is not meta
-        self.assertEqual(r.meta, meta)
         assert r.headers is not headers
         self.assertEqual(r.headers["caca"], "coco")
 
@@ -48,17 +43,12 @@ class BaseResponseTest(unittest.TestCase):
         """Test Response copy"""
 
         r1 = self.response_class("http://www.example.com", body="Some body")
-        r1.meta['foo'] = 'bar'
         r1.flags.append('cached')
         r2 = r1.copy()
 
         self.assertEqual(r1.status, r2.status)
         self.assertEqual(r1.body, r2.body)
 
-        # make sure meta dict is shallow copied
-        assert r1.meta is not r2.meta, "meta must be a shallow copy, not identical"
-        self.assertEqual(r1.meta, r2.meta)
-
         # make sure flags list is shallow copied
         assert r1.flags is not r2.flags, "flags must be a shallow copy, not identical"
         self.assertEqual(r1.flags, r2.flags)
@@ -67,6 +57,12 @@ class BaseResponseTest(unittest.TestCase):
         assert r1.headers is not r2.headers, "headers must be a shallow copy, not identical"
         self.assertEqual(r1.headers, r2.headers)
 
+    def test_copy_meta(self):
+        req = Request("http://www.example.com")
+        req.meta['foo'] = 'bar'
+        r1 = self.response_class("http://www.example.com", body="Some body", request=req)
+        assert r1.meta is req.meta
+
     def test_copy_inherited_classes(self):
         """Test Response children copies preserve their class"""
 
@@ -90,19 +86,11 @@ class BaseResponseTest(unittest.TestCase):
         self.assertEqual((r1.headers, r2.headers), ({}, hdrs))
 
         # Empty attributes (which may fail if not compared properly)
-        r3 = self.response_class("http://www.example.com", meta={'a': 1}, flags=['cached'])
-        r4 = r3.replace(body='', meta={}, flags=[])
+        r3 = self.response_class("http://www.example.com", flags=['cached'])
+        r4 = r3.replace(body='', flags=[])
         self.assertEqual(r4.body, '')
-        self.assertEqual(r4.meta, {})
         self.assertEqual(r4.flags, [])
 
-    def test_weakref_slots(self):
-        """Check that classes are using slots and are weak-referenceable"""
-        x = self.response_class('http://www.example.com')
-        weakref.ref(x)
-        assert not hasattr(x, '__dict__'), "%s does not use __slots__" % \
-            x.__class__.__name__
-
     def _assert_response_values(self, response, encoding, body):
         if isinstance(body, unicode):
             body_unicode = body
@@ -123,7 +111,7 @@ class ResponseText(BaseResponseTest):
 
     def test_no_unicode_url(self):
         self.assertRaises(TypeError, self.response_class, u'http://www.example.com')
-    
+
 
 class TextResponseTest(BaseResponseTest):
 
@@ -183,8 +171,8 @@ class TextResponseTest(BaseResponseTest):
         self.assertEqual(r2._headers_encoding(), None)
         self.assertEqual(r2._declared_encoding(), 'utf-8')
         self._assert_response_encoding(r2, 'utf-8')
-        self.assertEqual(r3._headers_encoding(), "iso-8859-1")
-        self.assertEqual(r3._declared_encoding(), "iso-8859-1")
+        self.assertEqual(r3._headers_encoding(), "cp1252")
+        self.assertEqual(r3._declared_encoding(), "cp1252")
         self.assertEqual(r4._headers_encoding(), None)
         self.assertEqual(r5._headers_encoding(), None)
         self._assert_response_encoding(r5, "utf-8")
@@ -200,19 +188,52 @@ class TextResponseTest(BaseResponseTest):
 
     def test_declared_encoding_invalid(self):
         """Check that unknown declared encodings are ignored"""
-        r = self.response_class("http://www.example.com", headers={"Content-type": ["text/html; charset=UKNOWN"]}, body="\xc2\xa3")
+        r = self.response_class("http://www.example.com",
+                                headers={"Content-type": ["text/html; charset=UKNOWN"]},
+                                body="\xc2\xa3")
         self.assertEqual(r._declared_encoding(), None)
         self._assert_response_values(r, 'utf-8', u"\xa3")
 
     def test_utf16(self):
         """Test utf-16 because UnicodeDammit is known to have problems with"""
-        r = self.response_class("http://www.example.com", body='\xff\xfeh\x00i\x00', encoding='utf-16')
+        r = self.response_class("http://www.example.com",
+                                body='\xff\xfeh\x00i\x00',
+                                encoding='utf-16')
         self._assert_response_values(r, 'utf-16', u"hi")
 
     def test_invalid_utf8_encoded_body_with_valid_utf8_BOM(self):
-        r6 = self.response_class("http://www.example.com", headers={"Content-type": ["text/html; charset=utf-8"]}, body="\xef\xbb\xbfWORD\xe3\xab")
+        r6 = self.response_class("http://www.example.com",
+                                 headers={"Content-type": ["text/html; charset=utf-8"]},
+                                 body="\xef\xbb\xbfWORD\xe3\xab")
         self.assertEqual(r6.encoding, 'utf-8')
-        self.assertEqual(r6.body_as_unicode(), u'\ufeffWORD\ufffd\ufffd')
+        self.assertEqual(r6.body_as_unicode(), u'WORD\ufffd\ufffd')
+
+    def test_bom_is_removed_from_body(self):
+        # Inferring encoding from body also cache decoded body as sideeffect,
+        # this test tries to ensure that calling response.encoding and
+        # response.body_as_unicode() in indistint order doesn't affect final
+        # values for encoding and decoded body.
+        url = 'http://example.com'
+        body = "\xef\xbb\xbfWORD"
+        headers = {"Content-type": ["text/html; charset=utf-8"]}
+
+        # Test response without content-type and BOM encoding
+        response = self.response_class(url, body=body)
+        self.assertEqual(response.encoding, 'utf-8')
+        self.assertEqual(response.body_as_unicode(), u'WORD')
+        response = self.response_class(url, body=body)
+        self.assertEqual(response.body_as_unicode(), u'WORD')
+        self.assertEqual(response.encoding, 'utf-8')
+
+        # Body caching sideeffect isn't triggered when encoding is declared in
+        # content-type header but BOM still need to be removed from decoded
+        # body
+        response = self.response_class(url, headers=headers, body=body)
+        self.assertEqual(response.encoding, 'utf-8')
+        self.assertEqual(response.body_as_unicode(), u'WORD')
+        response = self.response_class(url, headers=headers, body=body)
+        self.assertEqual(response.body_as_unicode(), u'WORD')
+        self.assertEqual(response.encoding, 'utf-8')
 
     def test_replace_wrong_encoding(self):
         """Test invalid chars are replaced properly"""
@@ -264,6 +285,10 @@ class HtmlResponseTest(TextResponseTest):
         r4 = r3.replace(body=body)
         self._assert_response_values(r4, 'iso-8859-1', body)
 
+    def test_html5_meta_charset(self):
+        body = """<html><head><meta charset="gb2312" /><title>Some page</title><body>bla bla</body>"""
+        r1 = self.response_class("http://www.example.com", body=body)
+        self._assert_response_values(r1, 'gb2312', body)
 
 
 class XmlResponseTest(TextResponseTest):
